package com.sun.org.apache.xml.internal.security.signature;

import java.security.AccessController;
import com.sun.org.slf4j.internal.LoggerFactory;
import com.sun.org.apache.xml.internal.security.c14n.InvalidCanonicalizerException;
import com.sun.org.apache.xml.internal.security.c14n.CanonicalizationException;
import java.util.HashMap;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolverSpi;
import java.util.Iterator;
import java.util.Set;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import org.w3c.dom.Node;
import java.util.Collections;
import com.sun.org.apache.xml.internal.security.transforms.Transforms;
import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import com.sun.org.apache.xml.internal.security.utils.I18n;
import com.sun.org.apache.xml.internal.security.utils.XMLUtils;
import com.sun.org.apache.xml.internal.security.exceptions.XMLSecurityException;
import java.util.ArrayList;
import org.w3c.dom.Document;
import com.sun.org.apache.xml.internal.security.utils.resolver.ResourceResolver;
import java.util.Map;
import org.w3c.dom.Element;
import java.util.List;
import com.sun.org.slf4j.internal.Logger;
import com.sun.org.apache.xml.internal.security.utils.SignatureElementProxy;

public class Manifest extends SignatureElementProxy
{
    public static final int MAXIMUM_REFERENCE_COUNT = 30;
    private static final Logger LOG;
    private static Integer referenceCount;
    private List<Reference> references;
    private Element[] referencesEl;
    private List<VerifiedReference> verificationResults;
    private Map<String, String> resolverProperties;
    private List<ResourceResolver> perManifestResolvers;
    private boolean secureValidation;
    
    public Manifest(Document doc) {
        super(doc);
        this.addReturnToSelf();
        this.references = new ArrayList();
    }
    
    public Manifest(Element element, String s) throws XMLSecurityException {
        this(element, s, true);
    }
    
    public Manifest(Element element, String baseURI, boolean secureValidation) throws XMLSecurityException {
        super(element, baseURI);
        Attr attributeNodeNS = element.getAttributeNodeNS(null, "Id");
        if (attributeNodeNS != null) {
            element.setIdAttributeNode(attributeNodeNS, true);
        }
        this.secureValidation = secureValidation;
        this.referencesEl = XMLUtils.selectDsNodes(this.getFirstChild(), "Reference");
        int length = this.referencesEl.length;
        if (length == 0) {
            throw new DOMException((short)4, I18n.translate("xml.WrongContent", new Object[] { "Reference", "Manifest" }));
        }
        if (secureValidation && length > (int)Manifest.referenceCount) {
            throw new XMLSecurityException("signature.tooManyReferences", new Object[] { Integer.valueOf(length), Manifest.referenceCount });
        }
        this.references = new ArrayList(length);
        for (int i = 0; i < length; ++i) {
            Element element2 = this.referencesEl[i];
            Attr attributeNodeNS2 = element2.getAttributeNodeNS(null, "Id");
            if (attributeNodeNS2 != null) {
                element2.setIdAttributeNode(attributeNodeNS2, true);
            }
            this.references.add(null);
        }
        return;
    }
    
    public void addDocument(String s, String s2, Transforms transforms, String s3, String id, String type) throws XMLSignatureException {
        Reference toAppend = new Reference(this.getDocument(), s, s2, this, transforms, s3);
        if (id != null) {
            toAppend.setId(id);
        }
        if (type != null) {
            toAppend.setType(type);
        }
        this.references.add(toAppend);
        this.appendSelf(toAppend);
        this.addReturnToSelf();
    }
    
    public void generateDigestValues() throws XMLSignatureException, ReferenceNotInitializedException {
        for (int i = 0; i < this.getLength(); ++i) {
            ((Reference)this.references.get(i)).generateDigestValue();
        }
    }
    
    public int getLength() {
        return this.references.size();
    }
    
    public Reference item(int n) throws XMLSecurityException {
        if (this.references.get(n) == null) {
            this.references.set(n, new Reference(this.referencesEl[n], this.baseURI, this, this.secureValidation));
        }
        return (Reference)this.references.get(n);
    }
    
    public void setId(String value) {
        if (value != null) {
            this.setLocalIdAttribute("Id", value);
        }
    }
    
    public String getId() {
        return this.getLocalAttribute("Id");
    }
    
    public boolean verifyReferences() throws MissingResourceFailureException, XMLSecurityException {
        return this.verifyReferences(false);
    }
    
    public boolean verifyReferences(boolean b) throws MissingResourceFailureException, XMLSecurityException {
        if (this.referencesEl == null) {
            this.referencesEl = XMLUtils.selectDsNodes(this.getFirstChild(), "Reference");
        }
        Manifest.LOG.debug("verify {} References", new Object[] { Integer.valueOf(this.referencesEl.length) });
        Manifest.LOG.debug("I am {} requested to follow nested Manifests", new Object[] { b ? "" : "not" });
        if (this.referencesEl.length == 0) {
            throw new XMLSecurityException("empty", new Object[] { "References are empty" });
        }
        if (this.secureValidation && this.referencesEl.length > (int)Manifest.referenceCount) {
            throw new XMLSecurityException("signature.tooManyReferences", new Object[] { Integer.valueOf(this.referencesEl.length), Manifest.referenceCount });
        }
        this.verificationResults = new ArrayList(this.referencesEl.length);
        boolean n = true;
        for (int i = 0; i < this.referencesEl.length; ++i) {
            Reference o = new Reference(this.referencesEl[i], this.baseURI, this, this.secureValidation);
            this.references.set(i, o);
            try {
                int verify = o.verify();
                if (verify == 0) {
                    n = false;
                }
                Manifest.LOG.debug("The Reference has Type {}", new Object[] { o.getType() });
                List<VerifiedReference> list = Collections.emptyList();
                if (n && b && o.typeIsReferenceToManifest()) {
                    Manifest.LOG.debug("We have to follow a nested Manifest");
                    try {
                        XMLSignatureInput dereferenceURIandPerformTransforms = o.dereferenceURIandPerformTransforms(null);
                        Set<Node> nodeSet = dereferenceURIandPerformTransforms.getNodeSet();
                        Object o2 = null;
                        for (Node node : nodeSet) {
                            if (node.getNodeType() == 1 && ((Element)node).getNamespaceURI().equals("http://www.w3.org/2000/09/xmldsig#") && ((Element)node).getLocalName().equals("Manifest")) {
                                try {
                                    o2 = new Manifest((Element)node, dereferenceURIandPerformTransforms.getSourceURI(), this.secureValidation);
                                    break;
                                }
                                catch (XMLSecurityException e) {
                                    Manifest.LOG.debug(e.getMessage(), e);
                                }
                            }
                            else {
                                continue;
                            }
                        }
                        if (o2 == null) {
                            throw new MissingResourceFailureException(o, "empty", new Object[] { "No Manifest found" });
                        }
                        o2.perManifestResolvers = this.perManifestResolvers;
                        o2.resolverProperties = this.resolverProperties;
                        if (!o2.verifyReferences(b)) {
                            n = false;
                            Manifest.LOG.warn("The nested Manifest was invalid (bad)");
                        }
                        else {
                            Manifest.LOG.debug("The nested Manifest was valid (good)");
                        }
                        list = o2.getVerificationResults();
                    }
                    catch (IOException ex) {
                        throw new ReferenceNotInitializedException(ex);
                    }
                    catch (ParserConfigurationException ex2) {
                        throw new ReferenceNotInitializedException(ex2);
                    }
                    catch (SAXException ex3) {
                        throw new ReferenceNotInitializedException(ex3);
                    }
                }
                this.verificationResults.add(new VerifiedReference((boolean)verify, o.getURI(), list));
            }
            catch (ReferenceNotInitializedException ex4) {
                throw new MissingResourceFailureException(ex4, o, "signature.Verification.Reference.NoInput", new Object[] { o.getURI() });
            }
        }
        return n;
    }
    
    public boolean getVerificationResult(int n) throws XMLSecurityException {
        if (n < 0 || n > this.getLength() - 1) {
            throw new XMLSecurityException(new IndexOutOfBoundsException(I18n.translate("signature.Verification.IndexOutOfBounds", new Object[] { Integer.toString(n), Integer.toString(this.getLength()) })));
        }
        if (this.verificationResults == null) {
            try {
                this.verifyReferences();
            }
            catch (Exception originalException) {
                throw new XMLSecurityException(originalException);
            }
        }
        return ((VerifiedReference)((ArrayList)this.verificationResults).get(n)).isValid();
    }
    
    public List<VerifiedReference> getVerificationResults() {
        if (this.verificationResults == null) {
            return Collections.emptyList();
        }
        return Collections.unmodifiableList(this.verificationResults);
    }
    
    public void addResourceResolver(ResourceResolver resourceResolver) {
        if (resourceResolver == null) {
            return;
        }
        if (this.perManifestResolvers == null) {
            this.perManifestResolvers = new ArrayList();
        }
        this.perManifestResolvers.add(resourceResolver);
        return;
    }
    
    public void addResourceResolver(ResourceResolverSpi resourceResolverSpi) {
        if (resourceResolverSpi == null) {
            return;
        }
        if (this.perManifestResolvers == null) {
            this.perManifestResolvers = new ArrayList();
        }
        this.perManifestResolvers.add(new ResourceResolver(resourceResolverSpi));
        return;
    }
    
    public List<ResourceResolver> getPerManifestResolvers() {
        return this.perManifestResolvers;
    }
    
    public Map<String, String> getResolverProperties() {
        return this.resolverProperties;
    }
    
    public void setResolverProperty(String s, String s2) {
        if (this.resolverProperties == null) {
            this.resolverProperties = new HashMap(10);
        }
        this.resolverProperties.put(s, s2);
    }
    
    public String getResolverProperty(String s) {
        return (String)this.resolverProperties.get(s);
    }
    
    public byte[] getSignedContentItem(int n) throws XMLSignatureException {
        try {
            return this.getReferencedContentAfterTransformsItem(n).getBytes();
        }
        catch (IOException ex) {
            throw new XMLSignatureException(ex);
        }
        catch (CanonicalizationException ex2) {
            throw new XMLSignatureException(ex2);
        }
        catch (InvalidCanonicalizerException ex3) {
            throw new XMLSignatureException(ex3);
        }
        catch (XMLSecurityException ex4) {
            throw new XMLSignatureException(ex4);
        }
    }
    
    public XMLSignatureInput getReferencedContentBeforeTransformsItem(int n) throws XMLSecurityException {
        return this.item(n).getContentsBeforeTransformation();
    }
    
    public XMLSignatureInput getReferencedContentAfterTransformsItem(int n) throws XMLSecurityException {
        return this.item(n).getContentsAfterTransformation();
    }
    
    public int getSignedContentLength() {
        return this.getLength();
    }
    
    @Override
    public String getBaseLocalName() {
        return "Manifest";
    }
    
    public boolean isSecureValidation() {
        return this.secureValidation;
    }
    
    static {
        LOG = LoggerFactory.getLogger(Manifest.class);
        Manifest.referenceCount = (Integer)AccessController.doPrivileged(()/*  */ -> Integer.valueOf(Integer.parseInt(System.getProperty("com.sun.org.apache.xml.internal.security.maxReferences", Integer.toString(30)))));
    }
}
